@@ -21,11 +21,14 @@ Functions related to reading/writing touchstones.
     hfss_touchstone_2_media
     hfss_touchstone_2_network
 """
-
+import re
 import os
 import zipfile
 import numpy
 import numpy as npy
+
+from six.moves import xrange
+
 from ..util import get_fid
 from ..network import Network
 from ..frequency import Frequency
@@ -38,9 +41,11 @@ class Touchstone:
     class to read touchstone s-parameter files
 
     The reference for writing this class is the draft of the
-    Touchstone(R) File Format Specification Rev 2.0 [#]_
+    Touchstone(R) File Format Specification Rev 2.0 [#]_ and
+    Touchstone(R) File Format Specification Version 2.0 [##]_
 
-    .. [#] http://www.eda-stds.org/ibis/adhoc/interconnect/touchstone_spec2_draft.pdf
+    .. [#] https://ibis.org/interconnect_wip/touchstone_spec2_draft.pdf
+    .. [##] https://ibis.org/touchstone_ver2.0/touchstone_ver2_0.pdf
     """
     def __init__(self, file):
         """
@@ -67,12 +72,15 @@ class Touchstone:
         ## file name of the touchstone data file
         self.filename = filename
 
-        ## file format version
+        ## file format version. 
+        # Defined by default to 1.0, since version number can be omitted in V1.0 format
         self.version = '1.0'
         ## comments in the file header
         self.comments = None
         ## unit of the frequency (Hz, kHz, MHz, GHz)
         self.frequency_unit = None
+        ## number of frequency points 
+        self.frequency_nb = None
         ## s-parameter type (S,Y,Z,G,H)
         self.parameter = None
         ## s-parameter format (MA, DB, RI)
@@ -92,6 +100,7 @@ class Touchstone:
         ## Store port names in a list if they exist in the file
         self.port_names = None
 
+        self.comment_variables=None
         self.load_file(fid)
 
     def load_file(self, fid):
@@ -101,13 +110,20 @@ class Touchstone:
 
         filename=self.filename
 
-        extention = filename.split('.')[-1].lower()
-        #self.rank = {'s1p':1, 's2p':2, 's3p':3, 's4p':4}.get(extention, None)
-        try:
-            self.rank = int(extention[1:-1])
-        except (ValueError):
-            raise (ValueError("filename does not have a s-parameter extension. It has  [%s] instead. please, correct the extension to of form: 'sNp', where N is any integer." %(extention)))
-
+        # Check the filename extension. 
+        # Should be .sNp for Touchstone format V1.0, and .ts for V2
+        extension = filename.split('.')[-1].lower()
+        
+        if (extension[0] == 's') and (extension[-1] == 'p'): # sNp
+            # check if N is a correct unmber
+            try:
+                self.rank = int(extension[1:-1])
+            except (ValueError):
+                raise (ValueError("filename does not have a s-parameter extension. It has  [%s] instead. please, correct the extension to of form: 'sNp', where N is any integer." %(extention)))
+        elif extension == 'ts':
+            pass
+        else:
+            raise Exception('Filename does not have the expected Touchstone extension (.sNp or .ts)')
 
         linenr = 0
         values = []
@@ -156,9 +172,32 @@ class Touchstone:
 
             # grab the [reference] string
             if line[:11] == '[reference]':
+                # The reference impedances can be span after the keyword  
+                # or on the following line
                 self.reference = [ float(r) for r in line.split()[2:] ]
+                if not self.reference:
+                    line = fid.readline()
+                    self.reference = [ float(r) for r in line.split()]
+                continue
+            
+            # grab the [Number of Ports] string
+            if line[:17] == '[number of ports]':
+                self.rank = int(line.split()[-1])
+                continue
+            
+            # grab the [Number of Frequencies] string
+            if line[:23] == '[number of frequencies]':
+                self.frequency_nb = line.split()[-1]
                 continue
 
+            # skip the [Network Data] keyword
+            if line[:14] == '[network data]':
+                continue
+            
+            # skip the [End] keyword
+            if line[:5] == '[end]':
+                continue
+            
             # the option line
             if line[0] == '#':
                 toks = line[1:].strip().split()
@@ -228,7 +267,24 @@ class Touchstone:
             if comment_line:
                 processed_comments = processed_comments + comment_line + '\n'
         return processed_comments
-
+    
+    def get_comment_variables(self):
+        '''
+        convert hfss variable comments to a dict of vars:(numbers,units)
+        '''
+        comments = self.comments
+        p1 = re.compile('\w* = \w*')
+        p2 = re.compile('\s*(\d*)\s*(\w*)')
+        var_dict = {}
+        for k in re.findall(p1, comments):
+            var, value = k.split('=')
+            var=var.rstrip()
+            try:
+                var_dict[var] = p2.match(value).groups()
+            except:
+                pass
+        return var_dict
+    
     def get_format(self, format="ri"):
         """
         returns the file format string used for the given format.
@@ -348,8 +404,55 @@ class Touchstone:
         noise_source_reflection = noise_values[:,2]
         noise_source_phase = noise_values[:,3]
         noise_normalized_resistance = noise_values[:,4]
+        
+    def is_from_hfss(self):
+        '''
+        Check if the Touchstone file has been produced by HFSS
+        
+        Returns
+        ------------
+        status : boolean
+            True if the Touchstone file has been produced by HFSS
+            False otherwise
+        '''    
+        status = False
+        if 'Exported from HFSS' in self.comments:
+            status = True
+        return status      
+    
+    def get_gamma_z0(self):
+        '''
+        Extracts Z0 and Gamma comments from touchstone file (is provided)
+        
+        Returns
+        --------
+        gamma : complex numpy.ndarray
+            complex  propagation constant
+        z0 : numpy.ndarray
+            complex port impedance    
+        '''
+        def line2ComplexVector(s):
+            return mf.scalar2Complex(npy.array([k for k in s.strip().split(' ')
+                                                if k != ''][self.rank*-2:],
+                                                dtype='float'))
+    
+        with open(self.filename) as f:
+            gamma, z0 = [],[]
+    
+            for line in f:
+                if '! Gamma' in line:
+                    gamma.append(line2ComplexVector(line.replace('! Gamma', '')))
+                if '! Port Impedance' in line:
+                    z0.append(line2ComplexVector(line.replace('! Port Impedance', '')))
+    
+            # If the file does not contain valid port impedance comments, set to default one
+            if len(z0) == 0:
+                z0 = self.resistance
+                #raise ValueError('Touchstone does not contain valid gamma, port impedance comments')
 
 
+        return npy.array(gamma), npy.array(z0)
+
 def hfss_touchstone_2_gamma_z0(filename):
     '''
     Extracts Z0 and Gamma comments from touchstone file
@@ -376,28 +479,9 @@ def hfss_touchstone_2_gamma_z0(filename):
     ----------
     >>> f,gamm,z0 = rf.hfss_touchstone_2_gamma_z0('line.s2p')
     '''
-    #TODO: make this work for different HFSS versions. and arbitrary
-    # number of ports
     ntwk = Network(filename)
 
-    def line2ComplexVector(s):
-        return mf.scalar2Complex(npy.array([k for k in s.strip().split(' ')
-                                            if k != ''][ntwk.nports*-2:],
-                                            dtype='float'))
-
-    with open(filename) as f:
-        gamma, z0 = [],[]
-
-        for line in f:
-            if '! Gamma' in line:
-                gamma.append(line2ComplexVector(line))
-            if '! Port Impedance' in line:
-                z0.append(line2ComplexVector(line))
-
-    if len(z0) == 0:
-        raise ValueError('Touchstone does not contain valid gamma, port impedance comments')
-
-    return ntwk.frequency.f, npy.array(gamma), npy.array(z0)
+    return ntwk.frequency.f, ntwk.gamma, ntwk.z0
 
 
 def hfss_touchstone_2_media(filename, f_unit='ghz'):
@@ -425,11 +509,12 @@ def hfss_touchstone_2_media(filename, f_unit='ghz'):
     ---------
     hfss_touchstone_2_gamma_z0 : returns gamma, and z0
     '''
-    f, gamma, z0 = hfss_touchstone_2_gamma_z0(filename)
-
-    freq = Frequency.from_f(f)
-    freq.unit = f_unit
+    ntwk = Network(filename)
 
+    freq = ntwk.frequency
+    gamma = ntwk.gamma
+    z0 = ntwk.z0
+    
 
     media_list = []
 
@@ -471,8 +556,6 @@ def hfss_touchstone_2_network(filename, f_unit='ghz'):
     hfss_touchstone_2_gamma_z0 : returns gamma, and z0
     '''
     my_network = Network(file=filename, f_unit=f_unit)
-    f,gamm,z0 = hfss_touchstone_2_gamma_z0(filename=filename)
-    my_network.z0 = z0
     return(my_network)
 
 
@@ -498,3 +581,4 @@ def read_zipped_touchstones(ziparchive, dir=""):
             network = Network.zipped_touchstone(fname, ziparchive)
             networks[network.name] = network
     return networks
+
